@@ -1,9 +1,15 @@
-from typing import Optional, Literal
+from typing import Optional, Literal, TypedDict
 from .StandardConstants import *
 from lxml import etree
 import datetime
 
 
+
+class InternetRelationDict(TypedDict):
+    value: INTERNET_RELATION_VALUE_LITERAL
+    bandwidth_mbitps: Optional[int]
+
+
 class TranslateableTag:
     def __init__(self) -> None:
         self._text = ""
@@ -285,7 +291,7 @@ class Screenshot:
 
         self.caption: TranslateableTag = TranslateableTag()
         "The caption"
-    
+
     def get_source_image(self) -> Optional[Image]:
         for image in self.images:
             if image.type in ("source", None):
@@ -331,16 +337,16 @@ class AppstreamComponent:
         self.project_license: str = ""
         "The project license"
 
-        self.urls: dict[str, str] = {}
+        self.urls: dict[URL_TYPES_LITERAL, str] = {}
         "The URLs"
 
-        self.oars: dict[str, str] = {}
+        self.oars: dict[OARS_ATTRIBUTE_TYPES_LITERAL, OARS_VALUE_TYPES_LITERAL] = {}
         "The content rating"
 
         self.categories: list[str] = []
         "The categories"
 
-        self.provides: dict[str, str] = {}
+        self.provides: dict[PROVIDES_TYPES_LITERAL, list[str]] = {}
         "The provides. The content of the depracted mimetype tag goes intp provides['mimetype']"
 
         self.releases: list[Release] = []
@@ -361,15 +367,26 @@ class AppstreamComponent:
         self.keywords: TranslateableList = TranslateableList()
         "The Keywords"
 
-        self.controls: dict[str, Optional[str]] = {}
+        self.controls: dict[CONTROL_TYPES_LITERAL, Optional[Literal["requires", "recommends", "supports"]]] = {}
         "The Controls"
 
+        self.internet: dict[Literal["requires", "recommends", "supports"], InternetRelationDict] = {}
+
         self.kudos: list[str] = []
         "The Kudos"
 
         self.update_contact: Optional[str] = None
         "The update contact"
 
+        self.replaces: list[str] = []
+        "The replaces tag"
+
+        self.suggests: list[str] = []
+        "The suggests tag"
+
+        self.custom: dict[str, str] = {}
+        "The custom tag"
+
         self.extends: list[str] = []
         "The extends tag for addons"
 
@@ -396,8 +413,12 @@ class AppstreamComponent:
         self.languages.clear()
         self.keywords.clear()
         self.controls.clear()
+        self.internet.clear()
         self.kudos.clear()
         self.update_contact = None
+        self.replaces.clear()
+        self.suggests.clear()
+        self.custom.clear()
         self.extends.clear()
 
         for i in PROVIDES_TYPES:
@@ -419,6 +440,17 @@ class AppstreamComponent:
             if i.text.strip() in CONTROL_TYPES:
                 self.controls[i.text.strip()] = relation
 
+        internet_tag = tag.find("internet")
+        if internet_tag is not None:
+            internet_dict: InternetRelationDict = {"value": internet_tag.text.strip()}
+
+            try:
+                internet_dict["bandwidth_mbitps"] = int(internet_tag.get("bandwidth_mbitps"))
+            except (ValueError, TypeError):
+                internet_dict["bandwidth_mbitps"] = None
+
+            self.internet[relation] = internet_dict
+
     def parse_component_tag(self, tag: etree._ElementTree) -> None:
         self.id = tag.find("id").text.strip()
 
@@ -498,7 +530,10 @@ class AppstreamComponent:
         for i in tag.findall("translation"):
             trans_dict = {}
             trans_dict["type"] = i.get("type")
-            trans_dict["value"] = i.text.strip()
+            if i.text is None:
+                trans_dict["value"] = ""
+            else:
+                trans_dict["value"] = i.text.strip()
             self.translation.append(trans_dict)
 
         languages_tag = tag.find("languages")
@@ -533,6 +568,21 @@ class AppstreamComponent:
         if update_contact_tag is not None:
             self.update_contact = update_contact_tag.text.strip()
 
+        replaces_tag = tag.find("replaces")
+        if replaces_tag is not None:
+            for i in replaces_tag.findall("id"):
+                self.replaces.append(i.text)
+
+        suggests_tag = tag.find("suggests")
+        if suggests_tag is not None:
+            for i in suggests_tag.findall("id"):
+                self.suggests.append(i.text)
+
+        custom_tag = tag.find("custom")
+        if custom_tag is not None:
+            for i in custom_tag.findall("value"):
+                self.custom[i.get("Key").strip()] = i.text.strip()
+
         for i in tag.findall("extends"):
             self.extends.append(i.text.strip())
 
@@ -555,9 +605,17 @@ class AppstreamComponent:
 
         for key, value in self.controls:
             if key in CONTROL_TYPES and value == relation:
-                control_tag = etree.SubElement("control")
+                control_tag = etree.SubElement(relation_tag, "control")
                 control_tag.text = key
 
+        if relation in self.internet:
+            internet_tag = etree.SubElement(relation_tag, "internet")
+
+            if self.internet[relation]["bandwidth_mbitps"] is not None:
+                internet_tag.set("bandwidth_mbitps", str(self.internet[relation]["bandwidth_mbitps"]))
+
+            internet_tag.text = self.internet[relation]["value"]
+
         if len(relation_tag.getchildren()) == 0:
             parent_tag.remove(relation_tag)
 
@@ -643,6 +701,25 @@ class AppstreamComponent:
             update_contact_tag = etree.SubElement(tag, "update_contact")
             update_contact_tag.text = self.update_contact
 
+        if len(self.replaces) > 0:
+            replaces_tag = etree.SubElement(tag, "replaces")
+            for i in self.replaces:
+                replaces_id_tag = etree.SubElement(replaces_tag, "id")
+                replaces_id_tag.text = i.strip()
+
+        if len(self.suggests) > 0:
+            suggests_tag = etree.SubElement(tag, "suggests")
+            for i in self.suggests:
+                suggests_id_tag = etree.SubElement(suggests_tag, "id")
+                suggests_id_tag.text = i.strip()
+
+        if len(self.custom) > 0:
+            custom_tag = etree.SubElement(tag, "custom")
+            for key, value in self.custom:
+                value_tag = etree.SubElement(custom_tag, "value")
+                value_tag.set("key", key.strip())
+                value_tag.text = value.strip()
+
         for i in self.extends:
             extends_tag = etree.SubElement(tag, "extends")
             extends_tag.text = i
